// Screen.cpp: implementation of the Screen class.
//
//////////////////////////////////////////////////////////////////////

#include "Screen.h"
#include "Gfx.h"
#include "Sound.h"
#include "Keyboard.h"
#include "Timers.h"
#include "Defs.h"
#include "TextureManager.h"
#include "Tokenizer.h"
#include "Utils.h"

void ScreenMouseCallback(unsigned long msg,unsigned long lparam,unsigned long uparam)
{
	SCREEN->mouse_callback(msg,lparam,uparam);
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Screen*	SCREEN=NULL;

Screen::Screen()
{
	_exit=false;
	if (!SCREEN) SCREEN=this;
	_sprite=NULL;
	_font=NULL;
	set_mouse_event_callback(ScreenMouseCallback);

	HWND desktop=GetDesktopWindow();
	RECT desktop_rect;
	GetWindowRect(desktop,&desktop_rect);

	_desktop_res.x=desktop_rect.right/1024.0f;
	_desktop_res.y=desktop_rect.bottom/768.0f;
}

Screen::~Screen()
{
	if (SCREEN==this) SCREEN=NULL;
	if (_sprite) _sprite->Release(); _sprite=NULL;
	if (_font) _font->Release(); _font=NULL; 
	set_mouse_event_callback(NULL);
	for (unsigned long i=0; i<_drawables.size(); i++)
	{
		if (_drawables[i])
		{
			delete _drawables[i];
			_drawables[i]=NULL;
		}
	}
	_drawables.clear();
}

long Screen::run()
{
	MSG				msg;
	unsigned long	t;

	unsigned long	lastTime;
	float			ms;

	_exit=false;

	lastTime=timeGetTime();

	while (!_exit)
	{
		ms=1000.0f/50.0f;

		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			// Stop app going more than a given frames per second
			do
			{
				t=timeGetTime();
			} while (t-lastTime<ms);
			run(float(t)-float(lastTime));
			lastTime=t;
		}
	}

	return _exit_code;
}

void Screen::run(float t)
{
	if (KEY) KEY->poll();
	if (TIMERS) TIMERS->poll();

	anim();
	if (!_exit)
	{
		render();

		GFX->swap_buffers();
	}
}

void Screen::load_assets(std::string filename)
{
	FILE* file=fopen(filename.c_str(),"rt");
	if (!file) 
	{
		ERROR_MSG("Can't find assets file!");
		return;
	}

	Tokenizer		tokens;
	char			buffer[1024];
	unsigned long	n;

	while (!feof(file))
	{
		fgets((char*)&buffer,1024,file);
		tokens.clear_tokens();
		n=tokens.tokenize((char*)&buffer);
		if (n>0)
		{
			char* type=tokens.get_token(0);
			if (!stricmp(type,"texture"))
			{
				char* internal_name=tokens.get_token(1);
				char* filename=tokens.get_token(2);

				TEXTURES->load(internal_name,filename);
			}
			else if (!stricmp(type,"sound"))
			{
				char* internal_name=tokens.get_token(1);
				char* filename=tokens.get_token(2);
				char* loops=tokens.get_token(3);
				bool loop;
				if (!stricmp(loops,"true")) loop=true;
				else loop=false;

				SOUND->load(internal_name,filename,false,loop);
			}
		}
	}

	fclose(file);
}

bool Screen::create_font()
{
	_font=NULL;
	HRESULT hRet=D3DXCreateFont(GFX->get_device(),
		32,0,
		FW_BOLD,
		0,
		FALSE,
		ANSI_CHARSET,
		OUT_DEFAULT_PRECIS,
		DEFAULT_QUALITY,
		DEFAULT_PITCH | FF_DONTCARE,
		"Comic Sans MS",&_font);
	if (hRet!=D3D_OK)
	{
		ERROR_MSG("Can't create font - D3D error!");
		return false;
	}

	hRet=D3DXCreateSprite(GFX->get_device(),&_sprite);
	if (hRet!=D3D_OK)
	{
		_font->Release();
		_font=NULL;
		return false;
	}

	return true;
}

void Screen::write_text(float x,float y,float r,float g,float b,float a,char *string)
{
	if (!_font) return;

	RECT		rect;
	D3DCOLOR	color;
	D3DXVECTOR3	c=GFX->get_color_modulation();

	color=D3DCOLOR_COLORVALUE(r*c.x,g*c.y,b*c.z,a);

	rect.left=long(x*_desktop_res.x);
	rect.top=long(y*_desktop_res.y);
//	rect.left=long(x);
//	rect.top=long(y);
	rect.bottom=long(y+1);
	rect.right=long(x+1);

	GFX->get_device()->BeginScene();
	GFX->get_device()->SetPixelShader(NULL);

	unsigned long f=DT_LEFT;

	_font->DrawText(NULL,string,-1,&rect,DT_CALCRECT|f,color);

	D3DXMATRIX	matrix;
//	D3DXMatrixTranslation(&matrix,0.0f,0.0f,0.5f);
	_sprite->Begin(D3DXSPRITE_ALPHABLEND|D3DXSPRITE_SORT_TEXTURE|D3DXSPRITE_SORT_DEPTH_BACKTOFRONT);
//	_sprite->SetTransform(&matrix);
	_font->DrawText(_sprite,string,-1,&rect,f,color);
	_sprite->End();
}

void Screen::mouse_callback(unsigned long msg,unsigned long lparam,unsigned long uparam)
{
}
