// GameScreen.cpp: implementation of the GameScreen class.
//
//////////////////////////////////////////////////////////////////////

#include "GameScreen.h"
#include "../framework/TextureManager.h"
#include "../framework/Gfx.h"
#include "../framework/BlitManager.h"
#include "../framework/Tokenizer.h"
#include "../framework/Keyboard.h"
#include "../framework/Utils.h"
#include "../framework/Sound.h"
#include "../framework/MouseCursor.h"
#include <algorithm>

#include "Ant.h"
#include "Grasshoper.h"
#include "Leaf.h"
#include "AntEater.h"
#include "AntHill.h"
#include "SelectionCircle.h"
#include "PhenoromeMap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GameScreen::GameScreen()
{

}

GameScreen::~GameScreen()
{
	BLIT->clear_cache();
	TEXTURES->clear_all();
	SOUND->clear_all();
}

bool GameScreen::init(long level)
{
	_level=level;

	if (!create_font()) return false;
	if (!read_assets()) return false;

	if (!load_level()) return false;

	_selected=NULL;
	_camera_focus=NULL;
	_last_frame_time=float(timeGetTime());

	add_drawable(new MouseCursor);
	add_drawable(new SelectionCircle);

	_pm=new PhenoromeMap;
	add_drawable(_pm);

	Sprite* clock=new Sprite(0.0f,0.0f,0.1f);
	clock->add_animation_frame(TEXTURES->find_by_name("Clock01"));
	clock->add_animation_frame(TEXTURES->find_by_name("Clock02"));
	clock->add_animation_frame(TEXTURES->find_by_name("Clock03"));
	clock->add_animation_frame(TEXTURES->find_by_name("Clock04"));
	clock->add_animation_frame(TEXTURES->find_by_name("Clock05"));
	clock->add_animation_frame(TEXTURES->find_by_name("Clock06"));
	clock->add_animation_frame(TEXTURES->find_by_name("Clock07"));
	clock->add_animation_frame(TEXTURES->find_by_name("Clock08"));
	clock->set_animation_loop(true);
	clock->set_animation_rate(500.0f);
	clock->set_scale(0.5f);
	add_drawable(clock);

	if (SOUND->get_sound_count()==0)
	{
		load_assets("data/SoundAssets.txt");
	}
	SOUND->stop_all();
	SOUND->play(SOUND->find_by_name("GameMusic"));

	return true;
}

bool GameScreen::read_assets()
{
	load_assets("data/GameAssets.txt");

	return true;
}

bool GameScreen::load_level()
{
	_n_ants=0;
	_grasshoper_rate=0;
	_ant_eater_rate=0;

	char filename[512];
	sprintf((char*)&filename,"data/Level%02li.txt",_level);
	FILE* file=fopen((char*)&filename,"rt");
	if (!file) 
	{
		ERROR_MSG("Can't find level def file!");
		return false;
	}

	// Load configuration file
	Tokenizer		tokens;
	char			buffer[1024];
	unsigned long	n;

	while (!feof(file))
	{
		fgets((char*)&buffer,1024,file);
		tokens.clear_tokens();
		n=tokens.tokenize((char*)&buffer);
		if (n>0)
		{
			char* type=tokens.get_token(0);
			if (!stricmp(type,"assets"))
			{
				char filename[512];
				
				sprintf((char*)&filename,"data/%s",tokens.get_token(1));

				load_assets((char*)&filename);
			}
			else if (!stricmp(type,"n_ants"))
			{
				_n_ants=atol(tokens.get_token(1));
			}
			else if (!stricmp(type,"leaf_rate"))
			{
				_leaf_rate=atof(tokens.get_token(1));
				_time_to_next_leaf=5000.0f;
			}
			else if (!stricmp(type,"decay_time"))
			{
				_decay_time=atof(tokens.get_token(1));
			}
			else if (!stricmp(type,"time_to_end"))
			{
				_time_to_end=atof(tokens.get_token(1))*1000;
			}
			else if (!stricmp(type,"grasshoper"))
			{
				_grasshoper_rate=atof(tokens.get_token(1));
				_time_to_next_grasshoper=_grasshoper_rate;
			}
			else if (!stricmp(type,"ant_eater"))
			{
				_ant_eater_rate=atof(tokens.get_token(1));
				_time_to_next_ant_eater=_ant_eater_rate;
			}
			else if (!stricmp(type,"ant_hill"))
			{
				float px=atof(tokens.get_token(1));
				float py=atof(tokens.get_token(2));

				AntHill* ah=new AntHill(px,py,0.6f);
				add_drawable(ah);
				_ant_hills.push_back(ah);
			}
		}
	}

	fclose(file);

	// Create objects
	for (unsigned long i=0; i<_n_ants; i++)
	{
		Ant* ant=new Ant(get_random_number(0,1024),get_random_number(0,768),0.5f);
		add_drawable(ant);
		_ants.push_back(ant);
	}

	return true;
}

void GameScreen::render()
{
	GFX->clear(D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,0.0f,0.0f,0.0f,0.0f,1.0f,0);
	GFX->set_2d_render();
	GFX->begin_scene();

	if (_camera_focus)
	{
		GFX->set_camera(_camera_focus->get_x_pos(),_camera_focus->get_y_pos(),0.0f);
	}
	else GFX->set_camera(512.0f,384.0f,0.0f);

	BLIT->add(TEXTURES->find_by_name("Background"),0.0f,0.0f,0.9f,1024.0f,768.0f,1.0f,1.0f,1.0f,1.0f);

	// Sort sprites
	std::sort(_drawables.begin(),_drawables.end(),PriorityOrderFunc());	
	// Render sprites
	for (size_t i=0; i<_drawables.size(); i++)
	{
		if (_drawables[i]) 
		{
			if (_drawables[i]->get_priority()<-50.0f) 
			{
				BLIT->render();
				BLIT->clear();
			}
			_drawables[i]->draw();
		}
	}

	BLIT->render();
	BLIT->clear();

	char buffer[512];
	sprintf((char*)&buffer,"%02li:%02li",int(_time_to_end/1000.0f)/60,int(_time_to_end/1000.0f)%60);
	write_text(70.0f,15.0f,0.0f,0.0f,0.0f,1.0f,(char*)&buffer);
	
	GFX->end_scene();
}

void GameScreen::anim()
{
	float t=timeGetTime()-_last_frame_time;
	_time_to_end-=t;
	if (_time_to_end<=0.0f)
	{
		set_exit(true,1);
	}
	if (t>50.0f) t=50.0f;

	_time_to_next_leaf-=t;
	if (_time_to_next_leaf<0.0f)
	{
		spawn_leaf();
	}
	if (_grasshoper_rate>0.0f)
	{
		_time_to_next_grasshoper-=t;
		if (_time_to_next_grasshoper<=0.0f)
		{
			_time_to_next_grasshoper=_grasshoper_rate;
			spawn_grasshoper();
		}
	}
	if (_ant_eater_rate>0.0f)
	{
		_time_to_next_ant_eater-=t;
		if (_time_to_next_ant_eater<=0.0f)
		{
			_time_to_next_ant_eater=_ant_eater_rate;
			spawn_ant_eater();
		}
	}

	if (KEY->is_pressed(DIK_ESCAPE))
	{
		set_exit(true,0);
	}

	for (size_t j=0; j<_drawables.size(); j++)
	{
		if (_drawables[j]) _drawables[j]->anim();
	}

	// Check if anthills are all full
	bool all_full=true;
	AntHills::iterator i;
	for (i=_ant_hills.begin(); i!=_ant_hills.end(); ++i)
	{
		AntHill* ah=*i;
		if (!ah->is_full())
		{
			all_full=false;
			break;
		}
	}
	if (all_full)
	{
		set_exit(true,2);
	}

	if (_ants.empty())
	{
		set_exit(true,1);
	}

	_last_frame_time=float(timeGetTime());
}

void GameScreen::shutdown()
{
	_ants.clear();
	_leafs.clear();
	_grasshopers.clear();
	_ant_eaters.clear();
	_ant_hills.clear();
	_pm=NULL;
}

void GameScreen::mouse_callback(unsigned long msg,unsigned long lparam,unsigned long uparam)
{
	switch (msg)
	{
		case WM_LBUTTONUP:
			{
				// Check if there is an ant near the cursor position
				D3DXVECTOR3	pt=CURSOR->get_pos();
				Ant* ant=get_ant(pt.x,pt.y,32.0f);
				if ((ant) && (ant!=_selected))
				{
					_selected=ant;
					SOUND->play(SOUND->find_by_name("AntSelect"));
				}
				else
				{
					if (_selected)
					{
						if ((_selected->get_state()!=ANT_RETURN) &&
							(_selected->get_state()!=ANT_FLEE))
						{
							_selected->go_to(pt.x,pt.y,2.0f);
						}
					}
				}
			}
			break;
		case WM_RBUTTONUP:
			{
				if (_selected)
				{
					_selected=NULL;
				}
			}
			break;
	}
}

Ant* GameScreen::get_ant(float x,float y,float radius)
{
	Ant*	sel=NULL;
	float	d_min=FLT_MAX;

	Ants::iterator i;
	for (i=_ants.begin(); i!=_ants.end(); ++i)
	{
		Ant* ant=*i;
		if (ant->get_state()!=ANT_RETURN)
		{
			float d=Sqr(ant->get_x_pos()-x)+Sqr(ant->get_y_pos()-y);
			if (d<Sqr(radius))
			{
				if (d<d_min)
				{
					d_min=d;
					sel=ant;
				}
			}
		}
	}
	return sel;
}

bool GameScreen::is_ant_on(float x,float y,Ant* exclude)
{
	Ants::iterator i;
	for (i=_ants.begin(); i!=_ants.end(); ++i)
	{
		Ant* ant=*i;
		if (ant!=exclude)
		{
			float d=Sqr(ant->get_x_pos()-x)+Sqr(ant->get_y_pos()-y);
			if (d<Sqr(16))
			{
				return true;
			}
		}
	}
	return false;
}

AntHill* GameScreen::find_nearest_anthill(float x,float y,float* distance)
{
	AntHill*	sel=NULL;
	float		d_min=FLT_MAX;

	AntHills::iterator i;
	for (i=_ant_hills.begin(); i!=_ant_hills.end(); ++i)
	{
		AntHill* ah=*i;
		if (!ah->is_full())
		{
			float d=Sqr(ah->get_x_pos()-x)+Sqr(ah->get_y_pos()-y);
			if (d<d_min)
			{
				sel=ah;
				d_min=d;
			}
		}
	}
	if (distance) *distance=d_min;
	return sel;
}

Grasshoper* GameScreen::find_nearest_grasshoper(float x,float y,float* distance)
{
	Grasshoper*	sel=NULL;
	float		d_min=FLT_MAX;

	Grasshopers::iterator i;
	for (i=_grasshopers.begin(); i!=_grasshopers.end(); ++i)
	{
		Grasshoper* ah=*i;
		float d=Sqr(ah->get_x_pos()-x)+Sqr(ah->get_y_pos()-y);
		if (d<d_min)
		{
			sel=ah;
			d_min=d;
		}
	}
	if (distance) *distance=d_min;
	return sel;
}

AntEater* GameScreen::find_nearest_ant_eater(float x,float y,float* distance)
{
	AntEater*	sel=NULL;
	float		d_min=FLT_MAX;

	AntEaters::iterator i;
	for (i=_ant_eaters.begin(); i!=_ant_eaters.end(); ++i)
	{
		AntEater* ah=*i;
		float d=Sqr(ah->get_x_pos()-x)+Sqr(ah->get_y_pos()-y);
		if (d<d_min)
		{
			sel=ah;
			d_min=d;
		}
	}
	if (distance) *distance=d_min;
	return sel;
}

void GameScreen::spawn_leaf()
{
	float px,py,d;
	do
	{
		px=get_random_number(40.0f,990.0f);
		py=get_random_number(40.0f,728.0f);
		find_nearest_anthill(px,py,&d);
	} while (d<Sqr(100.0f));

	Leaf* leaf=new Leaf(px,py,0.3f);
	add_drawable(leaf);
	_leafs.push_back(leaf);

	_time_to_next_leaf=_leaf_rate;
}

void GameScreen::spawn_grasshoper()
{
	Grasshoper* gh=new Grasshoper;
	add_drawable(gh);
	_grasshopers.push_back(gh);
}

void GameScreen::spawn_ant_eater()
{
	AntEater* gh=new AntEater;
	add_drawable(gh);
	_ant_eaters.push_back(gh);
}

Leaf* GameScreen::find_nearest_leaf(float x,float y,float* distance)
{
	Leaf*	sel=NULL;
	float	d_min=FLT_MAX;

	Leafs::iterator i;
	for (i=_leafs.begin(); i!=_leafs.end(); ++i)
	{
		Leaf* ah=*i;
		if (!ah->is_falling())
		{
			float d=Sqr(ah->get_x_pos()-x)+Sqr(ah->get_y_pos()-y);
			if (d<d_min)
			{
				sel=ah;
				d_min=d;
			}
		}
	}
	if (distance) *distance=d_min;
	return sel;
}

void GameScreen::remove_leaf(Leaf* leaf)
{
	Leafs::iterator i=_leafs.begin();
	while (i!=_leafs.end())
	{
		if ((*i)==leaf)
		{
			i=_leafs.erase(i);
		}
		else i++;
	}
}

void GameScreen::remove_grasshoper(Grasshoper* grasshoper)
{
	Grasshopers::iterator i=_grasshopers.begin();
	while (i!=_grasshopers.end())
	{
		if ((*i)==grasshoper)
		{
			i=_grasshopers.erase(i);
		}
		else i++;
	}
}

void GameScreen::remove_ant_eater(AntEater* ant_eater)
{
	AntEaters::iterator i=_ant_eaters.begin();
	while (i!=_ant_eaters.end())
	{
		if ((*i)==ant_eater)
		{
			i=_ant_eaters.erase(i);
		}
		else i++;
	}
}

void GameScreen::remove_ant(Ant* ant)
{
	Ants::iterator i=_ants.begin();
	while (i!=_ants.end())
	{
		if ((*i)==ant)
		{
			i=_ants.erase(i);
		}
		else i++;
	}
}
